@@ -133,6 +133,7 @@ option_strings = {
     'dyndns_update_ptr' : _("Whether the provider should explicitly update the PTR record as well"),
     'dyndns_force_tcp' : _("Whether the nsupdate utility should default to using TCP"),
     'dyndns_auth' : _("What kind of authentication should be used to perform the DNS update"),
+    'subdomain_enumerate' : _('Control enumeration of trusted domains'),
 
     # [provider/ipa]
     'ipa_domain' : _('IPA domain'),
@@ -309,6 +310,8 @@ option_strings = {
 
     'ldap_groups_use_matching_rule_in_chain' : _('Use LDAP_MATCHING_RULE_IN_CHAIN for group lookups'),
     'ldap_initgroups_use_matching_rule_in_chain' : _('Use LDAP_MATCHING_RULE_IN_CHAIN for initgroup lookups'),
+    'ldap_min_id' : _('Set lower boundary for allowed IDs from the LDAP server'),
+    'ldap_max_id' : _('Set upper boundary for allowed IDs from the LDAP server'),
 
     # [provider/ldap/auth]
     'ldap_pwd_policy' : _('Policy to evaluate the password expiration'),
