@@ -175,7 +175,7 @@ int sysdb_delete_recursive(struct sysdb_ctx *sysdb,
         goto done;
     }
 
-    DEBUG(9, ("Found [%d] items to delete.\n", msgs_count));
+    DEBUG(SSSDBG_TRACE_ALL, ("Found [%zu] items to delete.\n", msgs_count));
 
     qsort(msgs, msgs_count,
           sizeof(struct ldb_message *), compare_ldb_dn_comp_num);
@@ -233,8 +233,64 @@ int sysdb_search_entry(TALLOC_CTX *mem_ctx,
     return EOK;
 }
 
+/* =Search-Entry-by-SID-string============================================ */
 
-/* =Search-User-by-[UID/NAME]============================================= */
+int sysdb_search_entry_by_sid_str(TALLOC_CTX *mem_ctx,
+                                  struct sysdb_ctx *sysdb,
+                                  struct sss_domain_info *domain,
+                                  const char *search_base,
+                                  const char *filter_str,
+                                  const char *sid_str,
+                                  const char **attrs,
+                                  struct ldb_message **msg)
+{
+    TALLOC_CTX *tmp_ctx;
+    const char *def_attrs[] = { SYSDB_NAME, SYSDB_SID_STR, NULL };
+    struct ldb_message **msgs = NULL;
+    struct ldb_dn *basedn;
+    size_t msgs_count = 0;
+    char *filter;
+    int ret;
+
+    tmp_ctx = talloc_new(NULL);
+    if (!tmp_ctx) {
+        return ENOMEM;
+    }
+
+    basedn = ldb_dn_new_fmt(tmp_ctx, sysdb->ldb,
+                            search_base, domain->name);
+    if (!basedn) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    filter = talloc_asprintf(tmp_ctx, filter_str, sid_str);
+    if (!filter) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    ret = sysdb_search_entry(tmp_ctx, sysdb, basedn, LDB_SCOPE_SUBTREE, filter,
+                             attrs?attrs:def_attrs, &msgs_count, &msgs);
+    if (ret) {
+        goto done;
+    }
+
+    *msg = talloc_steal(mem_ctx, msgs[0]);
+
+done:
+    if (ret == ENOENT) {
+        DEBUG(SSSDBG_TRACE_FUNC, ("No such entry\n"));
+    }
+    else if (ret) {
+        DEBUG(SSSDBG_OP_FAILURE, ("Error: %d (%s)\n", ret, strerror(ret)));
+    }
+
+    talloc_zfree(tmp_ctx);
+    return ret;
+}
+
+/* =Search-User-by-[UID/SID/NAME]============================================= */
 
 int sysdb_search_user_by_name(TALLOC_CTX *mem_ctx,
                               struct sysdb_ctx *sysdb,
@@ -289,7 +345,7 @@ done:
         DEBUG(SSSDBG_TRACE_FUNC, ("No such entry\n"));
     }
     else if (ret) {
-        DEBUG(SSSDBG_TRACE_FUNC, ("Error: %d (%s)\n", ret, strerror(ret)));
+        DEBUG(SSSDBG_OP_FAILURE, ("Error: %d (%s)\n", ret, strerror(ret)));
     }
     talloc_zfree(tmp_ctx);
     return ret;
@@ -345,15 +401,28 @@ done:
         DEBUG(SSSDBG_TRACE_FUNC, ("No such entry\n"));
     }
     else if (ret) {
-        DEBUG(SSSDBG_TRACE_FUNC, ("Error: %d (%s)\n", ret, strerror(ret)));
+        DEBUG(SSSDBG_OP_FAILURE, ("Error: %d (%s)\n", ret, strerror(ret)));
     }
 
     talloc_zfree(tmp_ctx);
     return ret;
 }
 
+int sysdb_search_user_by_sid_str(TALLOC_CTX *mem_ctx,
+                                 struct sysdb_ctx *sysdb,
+                                 struct sss_domain_info *domain,
+                                 const char *sid_str,
+                                 const char **attrs,
+                                 struct ldb_message **msg)
+{
 
-/* =Search-Group-by-[GID/NAME]============================================ */
+   return sysdb_search_entry_by_sid_str(mem_ctx, sysdb, domain,
+                                        SYSDB_TMPL_USER_BASE,
+                                        SYSDB_PWSID_FILTER,
+                                        sid_str, attrs, msg);
+}
+
+/* =Search-Group-by-[GID/SID/NAME]============================================ */
 
 int sysdb_search_group_by_name(TALLOC_CTX *mem_ctx,
                                struct sysdb_ctx *sysdb,
@@ -393,7 +462,7 @@ done:
         DEBUG(SSSDBG_TRACE_FUNC, ("No such entry\n"));
     }
     else if (ret) {
-        DEBUG(SSSDBG_TRACE_FUNC, ("Error: %d (%s)\n", ret, strerror(ret)));
+        DEBUG(SSSDBG_OP_FAILURE, ("Error: %d (%s)\n", ret, strerror(ret)));
     }
     talloc_zfree(tmp_ctx);
     return ret;
@@ -407,7 +476,7 @@ int sysdb_search_group_by_gid(TALLOC_CTX *mem_ctx,
                               struct ldb_message **msg)
 {
     TALLOC_CTX *tmp_ctx;
-    const char *def_attrs[] = { SYSDB_NAME, SYSDB_UIDNUM, NULL };
+    const char *def_attrs[] = { SYSDB_NAME, SYSDB_GIDNUM, NULL };
     struct ldb_message **msgs = NULL;
     struct ldb_dn *basedn;
     size_t msgs_count = 0;
@@ -449,13 +518,26 @@ done:
         DEBUG(SSSDBG_TRACE_FUNC, ("No such entry\n"));
     }
     else if (ret) {
-        DEBUG(SSSDBG_TRACE_FUNC, ("Error: %d (%s)\n", ret, strerror(ret)));
+        DEBUG(SSSDBG_OP_FAILURE, ("Error: %d (%s)\n", ret, strerror(ret)));
     }
 
     talloc_zfree(tmp_ctx);
     return ret;
 }
 
+int sysdb_search_group_by_sid_str(TALLOC_CTX *mem_ctx,
+                                  struct sysdb_ctx *sysdb,
+                                  struct sss_domain_info *domain,
+                                  const char *sid_str,
+                                  const char **attrs,
+                                  struct ldb_message **msg)
+{
+
+   return sysdb_search_entry_by_sid_str(mem_ctx, sysdb, domain,
+                                        SYSDB_TMPL_GROUP_BASE,
+                                        SYSDB_GRSID_FILTER,
+                                        sid_str, attrs, msg);
+}
 
 /* =Search-Group-by-Name============================================ */
 
@@ -497,7 +579,7 @@ done:
         DEBUG(SSSDBG_TRACE_FUNC, ("No such entry\n"));
     }
     else if (ret) {
-        DEBUG(SSSDBG_TRACE_FUNC, ("Error: %d (%s)\n", ret, strerror(ret)));
+        DEBUG(SSSDBG_OP_FAILURE, ("Error: %d (%s)\n", ret, strerror(ret)));
     }
     talloc_zfree(tmp_ctx);
     return ret;
@@ -559,7 +641,7 @@ done:
         DEBUG(SSSDBG_TRACE_FUNC, ("No such entry\n"));
     }
     else if (ret) {
-        DEBUG(SSSDBG_TRACE_FUNC, ("Error: %d (%s)\n", ret, strerror(ret)));
+        DEBUG(SSSDBG_OP_FAILURE, ("Error: %d (%s)\n", ret, strerror(ret)));
     }
     talloc_zfree(tmp_ctx);
     return ret;
@@ -1116,15 +1198,17 @@ int sysdb_add_user(struct sysdb_ctx *sysdb,
 
     if (domain->id_max != 0 && uid != 0 &&
         (uid < domain->id_min || uid > domain->id_max)) {
-        DEBUG(2, ("Supplied uid [%d] is not in the allowed range [%d-%d].\n",
-                  uid, domain->id_min, domain->id_max));
+        DEBUG(SSSDBG_OP_FAILURE,
+              ("Supplied uid [%"SPRIuid"] is not in the allowed range "
+               "[%d-%d].\n", uid, domain->id_min, domain->id_max));
         return ERANGE;
     }
 
     if (domain->id_max != 0 && gid != 0 &&
         (gid < domain->id_min || gid > domain->id_max)) {
-        DEBUG(2, ("Supplied gid [%d] is not in the allowed range [%d-%d].\n",
-                  gid, domain->id_min, domain->id_max));
+        DEBUG(SSSDBG_OP_FAILURE,
+              ("Supplied gid [%"SPRIgid"] is not in the allowed range "
+               "[%d-%d].\n", gid, domain->id_min, domain->id_max));
         return ERANGE;
     }
 
@@ -1308,8 +1392,9 @@ int sysdb_add_group(struct sysdb_ctx *sysdb,
 
     if (domain->id_max != 0 && gid != 0 &&
         (gid < domain->id_min || gid > domain->id_max)) {
-        DEBUG(2, ("Supplied gid [%d] is not in the allowed range [%d-%d].\n",
-                  gid, domain->id_min, domain->id_max));
+        DEBUG(SSSDBG_OP_FAILURE,
+              ("Supplied gid [%"SPRIgid"] is not in the allowed range "
+               "[%d-%d].\n", gid, domain->id_min, domain->id_max));
         return ERANGE;
     }
 
@@ -1912,7 +1997,8 @@ sysdb_group_membership_mod(struct sysdb_ctx *sysdb,
                            const char *group,
                            const char *member,
                            enum sysdb_member_type type,
-                           int modify_op)
+                           int modify_op,
+                           bool is_dn)
 {
     struct ldb_dn *group_dn;
     struct ldb_dn *member_dn;
@@ -1936,7 +2022,12 @@ sysdb_group_membership_mod(struct sysdb_ctx *sysdb,
         goto done;
     }
 
-    group_dn = sysdb_group_dn(sysdb, tmp_ctx, domain, group);
+    if (!is_dn) {
+        group_dn = sysdb_group_dn(sysdb, tmp_ctx, domain, group);
+    } else {
+        group_dn = ldb_dn_new(tmp_ctx, sysdb->ldb, group);
+    }
+
     if (!group_dn) {
         ret = ENOMEM;
         goto done;
@@ -1953,10 +2044,11 @@ int sysdb_add_group_member(struct sysdb_ctx *sysdb,
                            struct sss_domain_info *domain,
                            const char *group,
                            const char *member,
-                           enum sysdb_member_type type)
+                           enum sysdb_member_type type,
+                           bool is_dn)
 {
-    return sysdb_group_membership_mod(sysdb, domain, group,
-                                      member, type, SYSDB_MOD_ADD);
+    return sysdb_group_membership_mod(sysdb, domain, group, member,
+                                      type, SYSDB_MOD_ADD, is_dn);
 }
 
 /* =Remove-member-from-Group(Native/Legacy)=============================== */
@@ -1966,10 +2058,11 @@ int sysdb_remove_group_member(struct sysdb_ctx *sysdb,
                               struct sss_domain_info *domain,
                               const char *group,
                               const char *member,
-                              enum sysdb_member_type type)
+                              enum sysdb_member_type type,
+                              bool is_dn)
 {
-    return sysdb_group_membership_mod(sysdb, domain, group,
-                                      member, type, SYSDB_MOD_DEL);
+    return sysdb_group_membership_mod(sysdb, domain, group, member,
+                                      type, SYSDB_MOD_DEL, is_dn);
 }
 
 
@@ -2367,7 +2460,7 @@ fail:
         DEBUG(SSSDBG_TRACE_FUNC, ("No such entry\n"));
     }
     else if (ret) {
-        DEBUG(SSSDBG_TRACE_FUNC, ("Error: %d (%s)\n", ret, strerror(ret)));
+        DEBUG(SSSDBG_OP_FAILURE, ("Error: %d (%s)\n", ret, strerror(ret)));
     }
     talloc_zfree(tmp_ctx);
     return ret;
@@ -2700,7 +2793,7 @@ fail:
     if (ret == ENOENT) {
         DEBUG(SSSDBG_TRACE_FUNC, ("Entry not found\n"));
     } else {
-        DEBUG(SSSDBG_TRACE_FUNC, ("Error: %d (%s)\n", ret, strerror(ret)));
+        DEBUG(SSSDBG_OP_FAILURE, ("Error: %d (%s)\n", ret, strerror(ret)));
     }
     talloc_zfree(tmp_ctx);
     return ret;
@@ -3031,12 +3124,13 @@ done:
     return ret;
 }
 
-errno_t sysdb_update_members(struct sysdb_ctx *sysdb,
-                             struct sss_domain_info *domain,
-                             const char *member,
-                             enum sysdb_member_type type,
-                             const char *const *add_groups,
-                             const char *const *del_groups)
+static errno_t sysdb_update_members_ex(struct sysdb_ctx *sysdb,
+                                       struct sss_domain_info *domain,
+                                       const char *member,
+                                       enum sysdb_member_type type,
+                                       const char *const *add_groups,
+                                       const char *const *del_groups,
+                                       bool is_dn)
 {
     errno_t ret;
     errno_t sret;
@@ -3059,8 +3153,8 @@ errno_t sysdb_update_members(struct sysdb_ctx *sysdb,
     if (add_groups) {
         /* Add the user to all add_groups */
         for (i = 0; add_groups[i]; i++) {
-            ret = sysdb_add_group_member(sysdb, domain,
-                                         add_groups[i], member, type);
+            ret = sysdb_add_group_member(sysdb, domain, add_groups[i],
+                                         member, type, is_dn);
             if (ret != EOK) {
                 DEBUG(1, ("Could not add member [%s] to group [%s]. "
                           "Skipping.\n", member, add_groups[i]));
@@ -3072,8 +3166,8 @@ errno_t sysdb_update_members(struct sysdb_ctx *sysdb,
     if (del_groups) {
         /* Remove the user from all del_groups */
         for (i = 0; del_groups[i]; i++) {
-            ret = sysdb_remove_group_member(sysdb, domain,
-                                            del_groups[i], member, type);
+            ret = sysdb_remove_group_member(sysdb, domain, del_groups[i],
+                                            member, type, is_dn);
             if (ret != EOK) {
                 DEBUG(1, ("Could not remove member [%s] from group [%s]. "
                           "Skipping\n", member, del_groups[i]));
@@ -3101,6 +3195,28 @@ done:
     return ret;
 }
 
+errno_t sysdb_update_members(struct sysdb_ctx *sysdb,
+                             struct sss_domain_info *domain,
+                             const char *member,
+                             enum sysdb_member_type type,
+                             const char *const *add_groups,
+                             const char *const *del_groups)
+{
+    return sysdb_update_members_ex(sysdb, domain, member, type,
+                                   add_groups, del_groups, false);
+}
+
+errno_t sysdb_update_members_dn(struct sysdb_ctx *sysdb,
+                                struct sss_domain_info *member_domain,
+                                const char *member,
+                                enum sysdb_member_type type,
+                                const char *const *add_groups,
+                                const char *const *del_groups)
+{
+    return sysdb_update_members_ex(sysdb, member_domain, member, type,
+                                   add_groups, del_groups, true);
+}
+
 errno_t sysdb_remove_attrs(struct sysdb_ctx *sysdb,
                            struct sss_domain_info *domain,
                            const char *name,
@@ -3243,7 +3359,7 @@ done:
     if (ret == ENOENT) {
         DEBUG(SSSDBG_TRACE_FUNC, ("No such entry.\n"));
     } else if (ret) {
-        DEBUG(SSSDBG_TRACE_FUNC, ("Error: %d (%s)\n", ret, strerror(ret)));
+        DEBUG(SSSDBG_OP_FAILURE, ("Error: %d (%s)\n", ret, strerror(ret)));
     }
 
     talloc_zfree(tmp_ctx);
