@@ -1642,7 +1642,7 @@ errno_t monitor_config_file_fallback(TALLOC_CTX *mem_ctx,
                                      monitor_reconf_fn fn,
                                      bool ignore_missing);
 
-#ifdef HAVE_SYS_INOTIFY_H
+#ifdef HAVE_INOTIFY
 static void process_config_file(struct tevent_context *ev,
                                 struct tevent_timer *te,
                                 struct timeval t, void *ptr);
@@ -1867,7 +1867,7 @@ static void rewatch_config_file(struct tevent_context *ev,
     talloc_free(rw_ctx);
     file_ctx->needs_update = 0;
 }
-#endif
+#endif /* HAVE_INOTIFY */
 
 static void poll_config_file(struct tevent_context *ev,
                                     struct tevent_timer *te,
@@ -1917,7 +1917,7 @@ static void poll_config_file(struct tevent_context *ev,
 static int try_inotify(struct config_file_ctx *file_ctx, const char *filename,
                        monitor_reconf_fn fn)
 {
-#ifdef HAVE_SYS_INOTIFY_H
+#ifdef HAVE_INOTIFY
     int err, fd_args, ret;
     struct tevent_fd *tfd;
     struct config_file_callback *cb;
@@ -1986,7 +1986,7 @@ static int try_inotify(struct config_file_ctx *file_ctx, const char *filename,
     return EOK;
 #else
     return EINVAL;
-#endif
+#endif /* HAVE_INOTIFY */
 }
 
 static int monitor_config_file(TALLOC_CTX *mem_ctx,
@@ -2695,7 +2695,8 @@ int main(int argc, const char *argv[])
 
     uid = getuid();
     if (uid != 0) {
-        DEBUG(SSSDBG_FATAL_FAILURE, ("Running under %d, must be root\n", uid));
+        DEBUG(SSSDBG_FATAL_FAILURE,
+              ("Running under %"SPRIuid", must be root\n", uid));
         sss_log(SSS_LOG_ALERT, "sssd must be run as root");
         return 8;
     }
