@@ -29,6 +29,214 @@ struct ad_server_data {
     bool gc;
 };
 
+errno_t ad_set_search_bases(struct sdap_options *id_opts);
+static errno_t ad_set_ad_id_options(struct ad_options *ad_opts,
+                                    struct sdap_options *id_opts);
+
+static struct sdap_options *
+ad_create_default_sdap_options(TALLOC_CTX *mem_ctx)
+{
+    struct sdap_options *id_opts;
+    errno_t ret;
+
+    id_opts = talloc_zero(mem_ctx, struct sdap_options);
+    if (!id_opts) {
+        return NULL;
+    }
+
+    ret = dp_copy_options(id_opts,
+                          ad_def_ldap_opts,
+                          SDAP_OPTS_BASIC,
+                          &id_opts->basic);
+    if (ret != EOK) {
+        goto fail;
+    }
+
+    /* Get sdap option maps */
+
+    /* General Attribute Map */
+    ret = sdap_copy_map(id_opts,
+                       ad_2008r2_attr_map,
+                       SDAP_AT_GENERAL,
+                       &id_opts->gen_map);
+    if (ret != EOK) {
+        goto fail;
+    }
+
+    /* User map */
+    ret = sdap_copy_map(id_opts,
+                       ad_2008r2_user_map,
+                       SDAP_OPTS_USER,
+                       &id_opts->user_map);
+    if (ret != EOK) {
+        goto fail;
+    }
+
+    /* Group map */
+    ret = sdap_copy_map(id_opts,
+                       ad_2008r2_group_map,
+                       SDAP_OPTS_GROUP,
+                       &id_opts->group_map);
+    if (ret != EOK) {
+        goto fail;
+    }
+
+    /* Netgroup map */
+    ret = sdap_copy_map(id_opts,
+                       ad_netgroup_map,
+                       SDAP_OPTS_NETGROUP,
+                       &id_opts->netgroup_map);
+    if (ret != EOK) {
+        goto fail;
+    }
+
+    /* Services map */
+    ret = sdap_copy_map(id_opts,
+                       ad_service_map,
+                       SDAP_OPTS_SERVICES,
+                       &id_opts->service_map);
+    if (ret != EOK) {
+        goto fail;
+    }
+
+    return id_opts;
+
+fail:
+    talloc_free(id_opts);
+    return NULL;
+}
+
+struct ad_options *
+ad_create_default_options(TALLOC_CTX *mem_ctx,
+                          const char *realm,
+                          const char *hostname)
+{
+    struct ad_options *ad_options;
+    errno_t ret;
+
+    ad_options = talloc_zero(mem_ctx, struct ad_options);
+    if (ad_options == NULL) return NULL;
+
+    ret = dp_copy_options(ad_options,
+                          ad_basic_opts,
+                          AD_OPTS_BASIC,
+                          &ad_options->basic);
+    if (ret != EOK) {
+        talloc_free(ad_options);
+        return NULL;
+    }
+
+    ad_options->id = ad_create_default_sdap_options(ad_options);
+    if (ad_options->id == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, ("Cannot initialize AD LDAP options\n"));
+        talloc_free(ad_options);
+        return NULL;
+    }
+
+    ret = dp_opt_set_string(ad_options->basic, AD_KRB5_REALM, realm);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, ("Cannot set AD domain\n"));
+        talloc_free(ad_options);
+        return NULL;
+    }
+
+    ret = dp_opt_set_string(ad_options->basic, AD_HOSTNAME, hostname);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, ("Cannot set AD domain\n"));
+        talloc_free(ad_options);
+        return NULL;
+    }
+
+    ret = ad_set_ad_id_options(ad_options, ad_options->id);
+    if (ret != EOK) {
+        talloc_free(ad_options);
+        return NULL;
+    }
+
+    return ad_options;
+}
+
+static errno_t
+ad_create_sdap_options(TALLOC_CTX *mem_ctx,
+                       struct confdb_ctx *cdb,
+                       const char *conf_path,
+                       struct sdap_options **_id_opts)
+{
+    struct sdap_options *id_opts;
+    errno_t ret;
+
+    id_opts = talloc_zero(mem_ctx, struct sdap_options);
+    if (!id_opts) {
+        ret = ENOMEM;
+        goto done;
+    }
+
+    ret = dp_get_options(id_opts, cdb, conf_path,
+                         ad_def_ldap_opts,
+                         SDAP_OPTS_BASIC,
+                         &id_opts->basic);
+    if (ret != EOK) {
+        goto done;
+    }
+
+    /* Get sdap option maps */
+
+    /* General Attribute Map */
+    ret = sdap_get_map(id_opts,
+                       cdb, conf_path,
+                       ad_2008r2_attr_map,
+                       SDAP_AT_GENERAL,
+                       &id_opts->gen_map);
+    if (ret != EOK) {
+        goto done;
+    }
+
+    /* User map */
+    ret = sdap_get_map(id_opts,
+                       cdb, conf_path,
+                       ad_2008r2_user_map,
+                       SDAP_OPTS_USER,
+                       &id_opts->user_map);
+    if (ret != EOK) {
+        goto done;
+    }
+
+    /* Group map */
+    ret = sdap_get_map(id_opts,
+                       cdb, conf_path,
+                       ad_2008r2_group_map,
+                       SDAP_OPTS_GROUP,
+                       &id_opts->group_map);
+    if (ret != EOK) {
+        goto done;
+    }
+
+    /* Netgroup map */
+    ret = sdap_get_map(id_opts,
+                       cdb, conf_path,
+                       ad_netgroup_map,
+                       SDAP_OPTS_NETGROUP,
+                       &id_opts->netgroup_map);
+    if (ret != EOK) {
+        goto done;
+    }
+
+    /* Services map */
+    ret = sdap_get_map(id_opts,
+                       cdb, conf_path,
+                       ad_service_map,
+                       SDAP_OPTS_SERVICES,
+                       &id_opts->service_map);
+    if (ret != EOK) {
+        goto done;
+    }
+
+    ret = EOK;
+    *_id_opts = id_opts;
+done:
+    return ret;
+}
+
 errno_t
 ad_get_common_options(TALLOC_CTX *mem_ctx,
                       struct confdb_ctx *cdb,
@@ -148,14 +356,15 @@ static errno_t
 _ad_servers_init(TALLOC_CTX *mem_ctx,
                  struct ad_service *service,
                  struct be_ctx *bectx,
+                 const char *fo_service,
+                 const char *fo_gc_service,
                  const char *servers,
-                 struct ad_options *options,
+                 const char *ad_domain,
                  bool primary)
 {
     size_t i;
     errno_t ret = 0;
     char **list;
-    char *ad_domain;
     struct ad_server_data *sdata;
     TALLOC_CTX *tmp_ctx;
 
@@ -169,8 +378,6 @@ _ad_servers_init(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-    ad_domain = dp_opt_get_string(options->basic, AD_DOMAIN);
-
     /* Add each of these servers to the failover service */
     for (i = 0; list[i]; i++) {
         if (be_fo_is_srv_identifier(list[i])) {
@@ -189,7 +396,7 @@ _ad_servers_init(TALLOC_CTX *mem_ctx,
             }
             sdata->gc = true;
 
-            ret = be_fo_add_srv_server(bectx, AD_GC_SERVICE_NAME, "gc",
+            ret = be_fo_add_srv_server(bectx, fo_gc_service, "gc",
                                        ad_domain, BE_FO_PROTO_TCP,
                                        false, sdata);
             if (ret != EOK) {
@@ -206,7 +413,7 @@ _ad_servers_init(TALLOC_CTX *mem_ctx,
             }
             sdata->gc = false;
 
-            ret = be_fo_add_srv_server(bectx, AD_SERVICE_NAME, "ldap",
+            ret = be_fo_add_srv_server(bectx, fo_service, "ldap",
                                        ad_domain, BE_FO_PROTO_TCP,
                                        false, sdata);
             if (ret != EOK) {
@@ -234,7 +441,7 @@ _ad_servers_init(TALLOC_CTX *mem_ctx,
         }
         sdata->gc = true;
 
-        ret = be_fo_add_server(bectx, AD_SERVICE_NAME, list[i], 0, sdata, primary);
+        ret = be_fo_add_server(bectx, fo_service, list[i], 0, sdata, primary);
         if (ret && ret != EEXIST) {
             DEBUG(SSSDBG_FATAL_FAILURE, ("Failed to add server\n"));
             goto done;
@@ -247,7 +454,7 @@ _ad_servers_init(TALLOC_CTX *mem_ctx,
         }
         sdata->gc = false;
 
-        ret = be_fo_add_server(bectx, AD_SERVICE_NAME, list[i], 0, sdata, primary);
+        ret = be_fo_add_server(bectx, fo_service, list[i], 0, sdata, primary);
         if (ret && ret != EEXIST) {
             DEBUG(SSSDBG_FATAL_FAILURE, ("Failed to add server\n"));
             goto done;
@@ -263,17 +470,21 @@ done:
 static inline errno_t
 ad_primary_servers_init(TALLOC_CTX *mem_ctx, struct ad_service *service,
                         struct be_ctx *bectx, const char *servers,
-                        struct ad_options *options)
+                        const char *fo_service, const char *fo_gc_service,
+                        const char *ad_domain)
 {
-    return _ad_servers_init(mem_ctx, service, bectx, servers, options, true);
+    return _ad_servers_init(mem_ctx, service, bectx, fo_service,
+                            fo_gc_service, servers, ad_domain, true);
 }
 
 static inline errno_t
 ad_backup_servers_init(TALLOC_CTX *mem_ctx, struct ad_service *service,
                         struct be_ctx *bectx, const char *servers,
-                        struct ad_options *options)
+                        const char *fo_service, const char *fo_gc_service,
+                        const char *ad_domain)
 {
-    return _ad_servers_init(mem_ctx, service, bectx, servers, options, false);
+    return _ad_servers_init(mem_ctx, service, bectx, fo_service,
+                            fo_gc_service, servers, ad_domain, false);
 }
 
 static int ad_user_data_cmp(void *ud1, void *ud2)
@@ -314,13 +525,15 @@ errno_t
 ad_failover_init(TALLOC_CTX *mem_ctx, struct be_ctx *bectx,
                  const char *primary_servers,
                  const char *backup_servers,
-                 struct ad_options *options,
+                 const char *krb5_realm,
+                 const char *ad_service,
+                 const char *ad_gc_service,
+                 const char *ad_domain,
                  struct ad_service **_service)
 {
     errno_t ret;
     TALLOC_CTX *tmp_ctx;
     struct ad_service *service;
-    char *realm;
 
     tmp_ctx = talloc_new(mem_ctx);
     if (!tmp_ctx) return ENOMEM;
@@ -338,8 +551,8 @@ ad_failover_init(TALLOC_CTX *mem_ctx, struct be_ctx *bectx,
         goto done;
     }
 
-    service->sdap->name = talloc_strdup(service->sdap, AD_SERVICE_NAME);
-    service->gc->name = talloc_strdup(service->gc, AD_GC_SERVICE_NAME);
+    service->sdap->name = talloc_strdup(service->sdap, ad_service);
+    service->gc->name = talloc_strdup(service->gc, ad_gc_service);
     if (!service->sdap->name || !service->gc->name) {
         ret = ENOMEM;
         goto done;
@@ -351,20 +564,20 @@ ad_failover_init(TALLOC_CTX *mem_ctx, struct be_ctx *bectx,
         goto done;
     }
 
-    ret = be_fo_add_service(bectx, AD_SERVICE_NAME, ad_user_data_cmp);
+    ret = be_fo_add_service(bectx, ad_service, ad_user_data_cmp);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to create failover service!\n"));
         goto done;
     }
 
-    ret = be_fo_add_service(bectx, AD_GC_SERVICE_NAME, ad_user_data_cmp);
+    ret = be_fo_add_service(bectx, ad_gc_service, ad_user_data_cmp);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to create GC failover service!\n"));
         goto done;
     }
 
     service->krb5_service->name = talloc_strdup(service->krb5_service,
-                                                AD_SERVICE_NAME);
+                                                ad_service);
     if (!service->krb5_service->name) {
         ret = ENOMEM;
         goto done;
@@ -372,14 +585,13 @@ ad_failover_init(TALLOC_CTX *mem_ctx, struct be_ctx *bectx,
     service->sdap->kinit_service_name = service->krb5_service->name;
     service->gc->kinit_service_name = service->krb5_service->name;
 
-    realm = dp_opt_get_string(options->basic, AD_KRB5_REALM);
-    if (!realm) {
+    if (!krb5_realm) {
         DEBUG(SSSDBG_CRIT_FAILURE, ("No Kerberos realm set\n"));
         ret = EINVAL;
         goto done;
     }
     service->krb5_service->realm =
-        talloc_strdup(service->krb5_service, realm);
+        talloc_strdup(service->krb5_service, krb5_realm);
     if (!service->krb5_service->realm) {
         ret = ENOMEM;
         goto done;
@@ -392,14 +604,16 @@ ad_failover_init(TALLOC_CTX *mem_ctx, struct be_ctx *bectx,
     }
 
     ret = ad_primary_servers_init(mem_ctx, service, bectx,
-                                  primary_servers, options);
+                                  primary_servers, ad_service,
+                                  ad_gc_service, ad_domain);
     if (ret != EOK) {
         goto done;
     }
 
     if (backup_servers) {
         ret = ad_backup_servers_init(mem_ctx, service, bectx,
-                                     backup_servers, options);
+                                     backup_servers, ad_service,
+                                     ad_gc_service, ad_domain);
         if (ret != EOK) {
             goto done;
         }
@@ -411,7 +625,7 @@ ad_failover_init(TALLOC_CTX *mem_ctx, struct be_ctx *bectx,
         return ret;
     }
 
-    ret = be_fo_service_add_callback(mem_ctx, bectx, AD_SERVICE_NAME,
+    ret = be_fo_service_add_callback(mem_ctx, bectx, ad_service,
                                      ad_resolve_callback, service);
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE,
@@ -419,7 +633,7 @@ ad_failover_init(TALLOC_CTX *mem_ctx, struct be_ctx *bectx,
         goto done;
     }
 
-    ret = be_fo_service_add_callback(mem_ctx, bectx, AD_GC_SERVICE_NAME,
+    ret = be_fo_service_add_callback(mem_ctx, bectx, ad_gc_service,
                                      ad_resolve_callback, service);
     if (ret != EOK) {
         DEBUG(SSSDBG_FATAL_FAILURE,
@@ -576,49 +790,14 @@ done:
     return;
 }
 
-errno_t
-ad_set_search_bases(struct sdap_options *id_opts);
-
-errno_t
-ad_get_id_options(struct ad_options *ad_opts,
-                   struct confdb_ctx *cdb,
-                   const char *conf_path,
-                   struct sdap_options **_opts)
+static errno_t
+ad_set_ad_id_options(struct ad_options *ad_opts,
+                     struct sdap_options *id_opts)
 {
     errno_t ret;
-    TALLOC_CTX *tmp_ctx;
-    struct sdap_options *id_opts;
     char *krb5_realm;
     char *keytab_path;
 
-    tmp_ctx = talloc_new(NULL);
-    if (!tmp_ctx) return ENOMEM;
-
-    id_opts = talloc_zero(tmp_ctx, struct sdap_options);
-    if (!id_opts) {
-        ret = ENOMEM;
-        goto done;
-    }
-
-    ret = sdap_domain_add(id_opts,
-                          ad_opts->id_ctx->sdap_id_ctx->be->domain,
-                          NULL);
-    if (ret != EOK) {
-        goto done;
-    }
-
-    ret = dp_get_options(id_opts, cdb, conf_path,
-                         ad_def_ldap_opts,
-                         SDAP_OPTS_BASIC,
-                         &id_opts->basic);
-    if (ret != EOK) {
-        goto done;
-    }
-
-    /* Set up search bases if they were assigned explicitly */
-    ret = ad_set_search_bases(id_opts);
-    if (ret != EOK) goto done;
-
     /* We only support Kerberos password policy with AD, so
      * force that on.
      */
@@ -671,66 +850,51 @@ ad_get_id_options(struct ad_options *ad_opts,
     /* fix schema to AD  */
     id_opts->schema_type = SDAP_SCHEMA_AD;
 
-    /* Get sdap option maps */
-
-    /* General Attribute Map */
-    ret = sdap_get_map(id_opts,
-                       cdb, conf_path,
-                       ad_2008r2_attr_map,
-                       SDAP_AT_GENERAL,
-                       &id_opts->gen_map);
-    if (ret != EOK) {
-        goto done;
-    }
-
-    /* User map */
-    ret = sdap_get_map(id_opts,
-                       cdb, conf_path,
-                       ad_2008r2_user_map,
-                       SDAP_OPTS_USER,
-                       &id_opts->user_map);
-    if (ret != EOK) {
-        goto done;
-    }
-
-    /* Group map */
-    ret = sdap_get_map(id_opts,
-                       cdb, conf_path,
-                       ad_2008r2_group_map,
-                       SDAP_OPTS_GROUP,
-                       &id_opts->group_map);
-    if (ret != EOK) {
-        goto done;
-    }
-
-    /* Netgroup map */
-    ret = sdap_get_map(id_opts,
-                       cdb, conf_path,
-                       ad_netgroup_map,
-                       SDAP_OPTS_NETGROUP,
-                       &id_opts->netgroup_map);
-    if (ret != EOK) {
-        goto done;
-    }
-
-    /* Services map */
-    ret = sdap_get_map(id_opts,
-                       cdb, conf_path,
-                       ad_service_map,
-                       SDAP_OPTS_SERVICES,
-                       &id_opts->service_map);
-    if (ret != EOK) {
-        goto done;
-    }
-
-    ad_opts->id = talloc_steal(ad_opts, id_opts);
-    *_opts = id_opts;
+    ad_opts->id = id_opts;
     ret = EOK;
 done:
-    talloc_free(tmp_ctx);
     return ret;
 }
 
+errno_t
+ad_get_id_options(struct ad_options *ad_opts,
+                  struct confdb_ctx *cdb,
+                  const char *conf_path,
+                  struct sdap_options **_opts)
+{
+    struct sdap_options *id_opts;
+    errno_t ret;
+
+    ret = ad_create_sdap_options(ad_opts, cdb, conf_path, &id_opts);
+    if (ret != EOK) {
+        return ENOMEM;
+    }
+
+    ret = ad_set_ad_id_options(ad_opts, id_opts);
+    if (ret != EOK) {
+        talloc_free(id_opts);
+        return ret;
+    }
+
+    ret = sdap_domain_add(id_opts,
+                          ad_opts->id_ctx->sdap_id_ctx->be->domain,
+                          NULL);
+    if (ret != EOK) {
+        talloc_free(id_opts);
+        return ret;
+    }
+
+    /* Set up search bases if they were assigned explicitly */
+    ret = ad_set_search_bases(id_opts);
+    if (ret != EOK) {
+        talloc_free(id_opts);
+        return ret;
+    }
+
+    *_opts = id_opts;
+    return EOK;
+}
+
 errno_t
 ad_set_search_bases(struct sdap_options *id_opts)
 {
@@ -898,3 +1062,33 @@ errno_t ad_get_dyndns_options(struct be_ctx *be_ctx,
 
     return EOK;
 }
+
+
+struct ad_id_ctx *
+ad_id_ctx_init(struct ad_options *ad_opts, struct be_ctx *bectx)
+{
+    struct sdap_id_ctx *sdap_ctx;
+    struct ad_id_ctx *ad_ctx;
+
+    ad_ctx = talloc_zero(ad_opts, struct ad_id_ctx);
+    if (ad_ctx == NULL) {
+        return NULL;
+    }
+    ad_ctx->ad_options = ad_opts;
+
+    sdap_ctx = sdap_id_ctx_new(ad_ctx, bectx, ad_opts->service->sdap);
+    if (sdap_ctx == NULL) {
+        talloc_free(ad_ctx);
+        return NULL;
+    }
+    ad_ctx->sdap_id_ctx = sdap_ctx;
+    ad_ctx->ldap_ctx = sdap_ctx->conn;
+
+    ad_ctx->gc_ctx = sdap_id_ctx_conn_add(sdap_ctx, ad_opts->service->gc);
+    if (ad_ctx->gc_ctx == NULL) {
+        talloc_free(ad_ctx);
+        return NULL;
+    }
+
+    return ad_ctx;
+}
