@@ -40,6 +40,7 @@
 #include "providers/ad/ad_srv.h"
 #include "providers/dp_dyndns.h"
 #include "providers/ad/ad_subdomains.h"
+#include "providers/ad/ad_domain_info.h"
 
 struct ad_options *ad_options = NULL;
 
@@ -204,7 +205,11 @@ sssm_ad_id_init(struct be_ctx *bectx,
         goto done;
     }
 
-    ret = ldap_id_setup_tasks(ad_ctx->sdap_id_ctx);
+    ret = sdap_id_setup_tasks(ad_ctx->sdap_id_ctx,
+                              ad_ctx->sdap_id_ctx->conn,
+                              ad_ctx->sdap_id_ctx->opts->sdom,
+                              ad_enumeration_send,
+                              ad_enumeration_recv);
     if (ret != EOK) {
         goto done;
     }
@@ -214,7 +219,6 @@ sssm_ad_id_init(struct be_ctx *bectx,
                           &ad_ctx->sdap_id_ctx->opts->idmap_ctx);
     if (ret != EOK) goto done;
 
-
     ret = setup_tls_config(ad_ctx->sdap_id_ctx->opts->basic);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE,
