@@ -24,6 +24,9 @@
 
 #include "providers/ldap/sdap_async.h"
 #include "providers/ad/ad_subdomains.h"
+#include "providers/ad/ad_domain_info.h"
+#include "providers/ldap/sdap_idmap.h"
+#include "util/util_sss_idmap.h"
 #include <ctype.h>
 #include <ndr.h>
 #include <ndr/ndr_nbt.h>
@@ -106,6 +109,7 @@ ad_subdom_store(struct ad_subdomains_ctx *ctx,
     struct ldb_message_element *el;
     char *sid_str;
     uint32_t trust_type;
+    bool mpg;
 
     tmp_ctx = talloc_new(NULL);
     if (tmp_ctx == NULL) {
@@ -156,9 +160,13 @@ ad_subdom_store(struct ad_subdomains_ctx *ctx,
         goto done;
     }
 
+    mpg = sdap_idmap_domain_has_algorithmic_mapping(
+                                             ctx->sdap_id_ctx->opts->idmap_ctx,
+                                             domain->domain_id);
+
     /* AD subdomains are currently all mpg and do not enumerate */
     ret = sysdb_subdomain_store(domain->sysdb, name, realm, flat, sid_str,
-                                true, false);
+                                mpg, false, NULL);
     if (ret != EOK) {
         DEBUG(SSSDBG_OP_FAILURE, ("sysdb_subdomain_store failed.\n"));
         goto done;
@@ -263,9 +271,7 @@ done:
 }
 
 static void ad_subdomains_get_conn_done(struct tevent_req *req);
-static errno_t ad_subdomains_get_master_sid(struct ad_subdomains_req_ctx *ctx);
-static void ad_subdomains_get_master_sid_done(struct tevent_req *req);
-static void ad_subdomains_get_netlogon_done(struct tevent_req *req);
+static void ad_subdomains_master_dom_done(struct tevent_req *req);
 static errno_t ad_subdomains_get_slave(struct ad_subdomains_req_ctx *ctx);
 
 static void ad_subdomains_retrieve(struct ad_subdomains_ctx *ctx,
@@ -340,236 +346,43 @@ static void ad_subdomains_get_conn_done(struct tevent_req *req)
         goto fail;
     }
 
-    ret = ad_subdomains_get_master_sid(ctx);
-    if (ret == EAGAIN) {
-        return;
-    } else if (ret != EOK) {
+    req = ad_master_domain_send(ctx, ctx->sd_ctx->be_ctx->ev,
+                                ctx->sd_ctx->ldap_ctx,
+                                ctx->sdap_op,
+                                ctx->sd_ctx->domain_name);
+    if (req == NULL) {
+        DEBUG(SSSDBG_OP_FAILURE, ("ad_master_domain_send failed.\n"));
+        ret = ENOMEM;
         goto fail;
     }
-
-    DEBUG(SSSDBG_OP_FAILURE, ("No search base available.\n"));
-    ret = EINVAL;
+    tevent_req_set_callback(req, ad_subdomains_master_dom_done, ctx);
+    return;
 
 fail:
     be_req_terminate(ctx->be_req, dp_error, ret, NULL);
 }
 
-static errno_t ad_subdomains_get_master_sid(struct ad_subdomains_req_ctx *ctx)
+static void ad_subdomains_master_dom_done(struct tevent_req *req)
 {
-    struct tevent_req *req;
-    struct sdap_search_base *base;
-    const char *master_sid_attrs[] = {AD_AT_OBJECT_SID, NULL};
-
-
-    base = ctx->sd_ctx->sdom->search_bases[ctx->base_iter];
-    if (base == NULL) {
-        return EOK;
-    }
-
-    req = sdap_get_generic_send(ctx, ctx->sd_ctx->be_ctx->ev,
-                           ctx->sd_ctx->sdap_id_ctx->opts,
-                           sdap_id_op_handle(ctx->sdap_op),
-                           base->basedn, LDAP_SCOPE_BASE,
-                           MASTER_DOMAIN_SID_FILTER, master_sid_attrs,
-                           NULL, 0,
-                           dp_opt_get_int(ctx->sd_ctx->sdap_id_ctx->opts->basic,
-                                          SDAP_SEARCH_TIMEOUT),
-                           false);
-
-    if (req == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE, ("sdap_get_generic_send failed.\n"));
-        return ENOMEM;
-    }
-
-    tevent_req_set_callback(req, ad_subdomains_get_master_sid_done, ctx);
-
-    return EAGAIN;
-}
-
-static void ad_subdomains_get_master_sid_done(struct tevent_req *req)
-{
-    int ret;
-    size_t reply_count;
-    struct sysdb_attrs **reply = NULL;
     struct ad_subdomains_req_ctx *ctx;
-    struct ldb_message_element *el;
-    char *sid_str;
-    enum idmap_error_code err;
-    static const char *attrs[] = {AD_AT_NETLOGON, NULL};
-    char *filter;
-    char *ntver;
+    errno_t ret;
 
     ctx = tevent_req_callback_data(req, struct ad_subdomains_req_ctx);
 
-    ret = sdap_get_generic_recv(req, ctx, &reply_count, &reply);
+    ret = ad_master_domain_recv(req, ctx,
+                                &ctx->flat_name, &ctx->master_sid);
     talloc_zfree(req);
     if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, ("sdap_get_generic_send request failed.\n"));
+        DEBUG(SSSDBG_OP_FAILURE, ("Cannot retrieve master domain info\n"));
         goto done;
     }
 
-    if (reply_count == 0) {
-        ctx->base_iter++;
-        ret = ad_subdomains_get_master_sid(ctx);
-        if (ret == EAGAIN) {
-            return;
-        } else if (ret != EOK) {
-            goto done;
-        }
-    } else if (reply_count == 1) {
-        ret = sysdb_attrs_get_el(reply[0], AD_AT_OBJECT_SID, &el);
-        if (ret != EOK || el->num_values != 1) {
-            DEBUG(SSSDBG_OP_FAILURE, ("sdap_attrs_get_el failed.\n"));
-            goto done;
-        }
-
-        err = sss_idmap_bin_sid_to_sid(ctx->sd_ctx->idmap_ctx,
-                                       el->values[0].data,
-                                       el->values[0].length,
-                                       &sid_str);
-        if (err != IDMAP_SUCCESS) {
-            DEBUG(SSSDBG_MINOR_FAILURE,
-                  ("Could not convert SID: [%s].\n", idmap_error_string(err)));
-            ret = EFAULT;
-            goto done;
-        }
-
-        ctx->master_sid = talloc_steal(ctx, sid_str);
-    } else {
-        DEBUG(SSSDBG_OP_FAILURE,
-              ("More than one result for domain SID found.\n"));
-        ret = EINVAL;
-        goto done;
-    }
-
-    DEBUG(SSSDBG_TRACE_FUNC, ("Found SID [%s].\n", ctx->master_sid));
-
-    ntver = sss_ldap_encode_ndr_uint32(ctx, NETLOGON_NT_VERSION_5EX |
-                                       NETLOGON_NT_VERSION_WITH_CLOSEST_SITE);
-    if (ntver == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE, ("sss_ldap_encode_ndr_uint32 failed.\n"));
-        ret = ENOMEM;
-        goto done;
-    }
-
-    filter = talloc_asprintf(ctx, "(&(%s=%s)(%s=%s))",
-                             AD_AT_DNS_DOMAIN, ctx->sd_ctx->domain_name,
-                             AD_AT_NT_VERSION, ntver);
-    if (filter == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE, ("talloc_asprintf failed.\n"));
-        ret = ENOMEM;
-        goto done;
-    }
-
-    req = sdap_get_generic_send(ctx, ctx->sd_ctx->be_ctx->ev,
-                           ctx->sd_ctx->sdap_id_ctx->opts,
-                           sdap_id_op_handle(ctx->sdap_op),
-                           "", LDAP_SCOPE_BASE, filter, attrs, NULL, 0,
-                           dp_opt_get_int(ctx->sd_ctx->sdap_id_ctx->opts->basic,
-                                          SDAP_SEARCH_TIMEOUT),
-                           false);
-    if (req == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE, ("sdap_get_generic_send failed.\n"));
-        ret = ENOMEM;
-        goto done;
-    }
-
-    tevent_req_set_callback(req, ad_subdomains_get_netlogon_done, ctx);
-    return;
-
-done:
-    be_req_terminate(ctx->be_req, DP_ERR_FATAL, ret, NULL);
-}
-
-static void ad_subdomains_get_netlogon_done(struct tevent_req *req)
-{
-    int ret;
-    size_t reply_count;
-    struct sysdb_attrs **reply = NULL;
-    struct ad_subdomains_req_ctx *ctx;
-    struct ldb_message_element *el;
-    DATA_BLOB blob;
-    enum ndr_err_code ndr_err;
-    struct ndr_pull *ndr_pull = NULL;
-    struct netlogon_samlogon_response response;
-    int dp_error = DP_ERR_FATAL;
-
-    ctx = tevent_req_callback_data(req, struct ad_subdomains_req_ctx);
-
-    ret = sdap_get_generic_recv(req, ctx, &reply_count, &reply);
-    talloc_zfree(req);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, ("sdap_get_generic_send request failed.\n"));
-        goto done;
-    }
-
-    if (reply_count == 0) {
-        DEBUG(SSSDBG_TRACE_FUNC, ("No netlogon data available.\n"));
-        ret = ENOENT;
-        goto done;
-    } else if (reply_count > 1) {
-        DEBUG(SSSDBG_OP_FAILURE,
-              ("More than one netlogon info returned.\n"));
-        ret = EINVAL;
-        goto done;
-    }
-
-    ret = sysdb_attrs_get_el(reply[0], AD_AT_NETLOGON, &el);
-    if (ret != EOK) {
-        DEBUG(SSSDBG_OP_FAILURE, ("sysdb_attrs_get_el() failed\n"));
-        goto done;
-    }
-
-    if (el->num_values == 0) {
-        DEBUG(SSSDBG_OP_FAILURE, ("netlogon has no value\n"));
-        ret = ENOENT;
-        goto done;
-    } else if (el->num_values > 1) {
-        DEBUG(SSSDBG_OP_FAILURE, ("More than one netlogon value?\n"));
-        ret = EIO;
-        goto done;
-    }
-
-    blob.data =  el->values[0].data;
-    blob.length = el->values[0].length;
-
-    ndr_pull = ndr_pull_init_blob(&blob, ctx);
-    if (ndr_pull == NULL) {
-        DEBUG(SSSDBG_OP_FAILURE, ("ndr_pull_init_blob() failed.\n"));
-        ret = ENOMEM;
-        goto done;
-    }
-
-    ndr_err = ndr_pull_netlogon_samlogon_response(ndr_pull, NDR_SCALARS,
-                                                  &response);
-    if (!NDR_ERR_CODE_IS_SUCCESS(ndr_err)) {
-        DEBUG(SSSDBG_OP_FAILURE, ("ndr_pull_netlogon_samlogon_response() "
-                                  "failed [%d]\n", ndr_err));
-        ret = EBADMSG;
-        goto done;
-    }
-
-    if (!(response.ntver & NETLOGON_NT_VERSION_5EX)) {
-        DEBUG(SSSDBG_OP_FAILURE, ("Wrong version returned [%x]\n",
-                                  response.ntver));
-        ret = EBADMSG;
-        goto done;
-    }
-
-    if (response.data.nt5_ex.domain_name != NULL &&
-        *response.data.nt5_ex.domain_name != '\0') {
-        ctx->flat_name = talloc_strdup(ctx, response.data.nt5_ex.domain_name);
-        if (ctx->flat_name == NULL) {
-            DEBUG(SSSDBG_OP_FAILURE, ("talloc_strdup failed.\n"));
-            ret = ENOMEM;
-            goto done;
-        }
-    }
-
-    DEBUG(SSSDBG_TRACE_FUNC, ("Found flat name [%s].\n", ctx->flat_name));
-
     ret = sysdb_master_domain_add_info(ctx->sd_ctx->be_ctx->domain,
                                        ctx->flat_name, ctx->master_sid);
+    if (ret != EOK) {
+        DEBUG(SSSDBG_OP_FAILURE, ("Cannot save master domain info\n"));
+        goto done;
+    }
 
     ret = ad_subdomains_get_slave(ctx);
     if (ret == EAGAIN) {
@@ -579,7 +392,7 @@ static void ad_subdomains_get_netlogon_done(struct tevent_req *req)
     }
 
 done:
-    be_req_terminate(ctx->be_req, dp_error, ret, NULL);
+    be_req_terminate(ctx->be_req, DP_ERR_FATAL, ret, NULL);
 }
 
 static void ad_subdomains_get_slave_domain_done(struct tevent_req *req);
@@ -675,7 +488,7 @@ static void ad_subdomains_get_slave_domain_done(struct tevent_req *req)
             goto done;
         }
 
-        ret = sss_write_domain_mappings(ctx->sd_ctx->be_ctx->domain);
+        ret = sss_write_domain_mappings(ctx->sd_ctx->be_ctx->domain, false);
         if (ret != EOK) {
             DEBUG(SSSDBG_MINOR_FAILURE,
                   ("sss_krb5_write_mappings failed.\n"));
@@ -777,16 +590,6 @@ struct bet_ops ad_subdomains_ops = {
     .finalize = NULL
 };
 
-static void *idmap_talloc(size_t size, void *pvt)
-{
-    return talloc_size(pvt, size);
-}
-
-static void idmap_free(void *ptr, void *pvt)
-{
-    talloc_free(ptr);
-}
-
 int ad_subdom_init(struct be_ctx *be_ctx,
                    struct ad_id_ctx *id_ctx,
                    const char *ad_domain,
@@ -825,7 +628,8 @@ int ad_subdom_init(struct be_ctx *be_ctx,
         DEBUG(SSSDBG_MINOR_FAILURE, ("Failed to add subdom offline callback"));
     }
 
-    err = sss_idmap_init(idmap_talloc, ctx, idmap_free, &ctx->idmap_ctx);
+    err = sss_idmap_init(sss_idmap_talloc, ctx, sss_idmap_talloc_free,
+                         &ctx->idmap_ctx);
     if (err != IDMAP_SUCCESS) {
         DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to initialize idmap context.\n"));
         return EFAULT;
