@@ -39,16 +39,16 @@ static void ipa_access_reply(struct hbac_ctx *hbac_ctx, int pam_status)
 {
     struct be_req *be_req = hbac_ctx->be_req;
     struct pam_data *pd;
-    pd = talloc_get_type(be_req->req_data, struct pam_data);
+    pd = talloc_get_type(be_req_get_data(be_req), struct pam_data);
     pd->pam_status = pam_status;
 
     /* destroy HBAC context now to release all used resources and LDAP connection */
     talloc_zfree(hbac_ctx);
 
     if (pam_status == PAM_SUCCESS || pam_status == PAM_PERM_DENIED) {
-        be_req->fn(be_req, DP_ERR_OK, pam_status, NULL);
+        be_req_terminate(be_req, DP_ERR_OK, pam_status, NULL);
     } else {
-        be_req->fn(be_req, DP_ERR_FATAL, pam_status, NULL);
+        be_req_terminate(be_req, DP_ERR_FATAL, pam_status, NULL);
     }
 }
 
@@ -78,25 +78,22 @@ void ipa_access_handler(struct be_req *be_req)
     struct pam_data *pd;
     struct ipa_access_ctx *ipa_access_ctx;
     struct tevent_req *req;
+    struct be_ctx *be_ctx = be_req_get_be_ctx(be_req);
 
-    pd = talloc_get_type(be_req->req_data, struct pam_data);
+    pd = talloc_get_type(be_req_get_data(be_req), struct pam_data);
 
-    ipa_access_ctx = talloc_get_type(
-                              be_req->be_ctx->bet_info[BET_ACCESS].pvt_bet_data,
-                              struct ipa_access_ctx);
+    ipa_access_ctx = talloc_get_type(be_ctx->bet_info[BET_ACCESS].pvt_bet_data,
+                                     struct ipa_access_ctx);
 
     /* First, verify that this account isn't locked.
      * We need to do this in case the auth phase was
      * skipped (such as during GSSAPI single-sign-on
      * or SSH public key exchange.
      */
-    req = sdap_access_send(be_req,
-                           be_req->be_ctx->ev,
-                           be_req,
-                           ipa_access_ctx->sdap_access_ctx,
-                           pd);
+    req = sdap_access_send(be_req, be_ctx->ev, be_ctx, be_ctx->domain,
+                           ipa_access_ctx->sdap_access_ctx, pd);
     if (!req) {
-        be_req->fn(be_req, DP_ERR_FATAL, PAM_SYSTEM_ERR, NULL);
+        be_req_terminate(be_req, DP_ERR_FATAL, PAM_SYSTEM_ERR, NULL);
         return;
     }
     tevent_req_set_callback(req, ipa_hbac_check, be_req);
@@ -105,51 +102,56 @@ void ipa_access_handler(struct be_req *be_req)
 static void ipa_hbac_check(struct tevent_req *req)
 {
     struct be_req *be_req;
+    struct be_ctx *be_ctx;
     struct pam_data *pd;
     struct hbac_ctx *hbac_ctx = NULL;
     const char *deny_method;
-    int pam_status = PAM_SYSTEM_ERR;
     struct ipa_access_ctx *ipa_access_ctx;
     int ret;
 
     be_req = tevent_req_callback_data(req, struct be_req);
-    pd = talloc_get_type(be_req->req_data, struct pam_data);
+    be_ctx = be_req_get_be_ctx(be_req);
+    pd = talloc_get_type(be_req_get_data(be_req), struct pam_data);
 
-    ret = sdap_access_recv(req, &pam_status);
+    ret = sdap_access_recv(req);
     talloc_zfree(req);
-    if (ret != EOK) goto fail;
 
-    switch(pam_status) {
-    case PAM_SUCCESS:
+    switch(ret) {
+    case EOK:
         /* Account wasn't locked. Continue below
          * to HBAC processing.
          */
         break;
-    case PAM_PERM_DENIED:
+    case ERR_ACCESS_DENIED:
         /* Account was locked. Return permission denied
          * here.
          */
         pd->pam_status = PAM_PERM_DENIED;
-        be_req->fn(be_req, DP_ERR_OK, PAM_PERM_DENIED, NULL);
+        be_req_terminate(be_req, DP_ERR_OK, pd->pam_status, NULL);
+        return;
+    case ERR_ACCOUNT_EXPIRED:
+        pd->pam_status = PAM_ACCT_EXPIRED;
+        be_req_terminate(be_req, DP_ERR_OK, pd->pam_status, NULL);
         return;
     default:
         /* We got an unexpected error. Return it as-is */
         pd->pam_status = PAM_SYSTEM_ERR;
-        be_req->fn(be_req, DP_ERR_FATAL, pam_status, NULL);
+        be_req_terminate(be_req, DP_ERR_FATAL, pd->pam_status,
+                         sss_strerror(ret));
         return;
     }
 
     hbac_ctx = talloc_zero(be_req, struct hbac_ctx);
     if (hbac_ctx == NULL) {
         DEBUG(1, ("talloc failed.\n"));
+        ret = ENOMEM;
         goto fail;
     }
 
     hbac_ctx->be_req = be_req;
     hbac_ctx->pd = pd;
-    ipa_access_ctx = talloc_get_type(
-                              be_req->be_ctx->bet_info[BET_ACCESS].pvt_bet_data,
-                              struct ipa_access_ctx);
+    ipa_access_ctx = talloc_get_type(be_ctx->bet_info[BET_ACCESS].pvt_bet_data,
+                                     struct ipa_access_ctx);
     hbac_ctx->access_ctx = ipa_access_ctx;
     hbac_ctx->sdap_ctx = ipa_access_ctx->sdap_ctx;
     hbac_ctx->ipa_options = ipa_access_ctx->ipa_options;
@@ -157,6 +159,7 @@ static void ipa_hbac_check(struct tevent_req *req)
     hbac_ctx->search_bases = ipa_access_ctx->hbac_search_bases;
     if (hbac_ctx->search_bases == NULL) {
         DEBUG(1, ("No HBAC search base found.\n"));
+        ret = EINVAL;
         goto fail;
     }
 
@@ -178,9 +181,9 @@ static void ipa_hbac_check(struct tevent_req *req)
 fail:
     if (hbac_ctx) {
         /* Return an proper error */
-        ipa_access_reply(hbac_ctx, pam_status);
+        ipa_access_reply(hbac_ctx, PAM_SYSTEM_ERR);
     } else {
-        be_req->fn(be_req, DP_ERR_FATAL, pam_status, NULL);
+        be_req_terminate(be_req, DP_ERR_FATAL, PAM_SYSTEM_ERR, NULL);
     }
 }
 
@@ -191,8 +194,9 @@ static int hbac_retry(struct hbac_ctx *hbac_ctx)
     bool offline;
     time_t now, refresh_interval;
     struct ipa_access_ctx *access_ctx = hbac_ctx->access_ctx;
+    struct be_ctx *be_ctx = be_req_get_be_ctx(hbac_ctx->be_req);
 
-    offline = be_is_offline(hbac_ctx->be_req->be_ctx);
+    offline = be_is_offline(be_ctx);
     DEBUG(9, ("Connection status is [%s].\n", offline ? "offline" : "online"));
 
     refresh_interval = dp_opt_get_int(hbac_ctx->ipa_options,
@@ -208,14 +212,14 @@ static int hbac_retry(struct hbac_ctx *hbac_ctx)
     if (!offline) {
         if (hbac_ctx->sdap_op == NULL) {
             hbac_ctx->sdap_op = sdap_id_op_create(hbac_ctx,
-                                        hbac_ctx_sdap_id_ctx(hbac_ctx)->conn_cache);
+                                          hbac_ctx->sdap_ctx->conn->conn_cache);
             if (hbac_ctx->sdap_op == NULL) {
                 DEBUG(1, ("sdap_id_op_create failed.\n"));
                 return EIO;
             }
         }
 
-        subreq = sdap_id_op_connect_send(hbac_ctx_sdap_id_op(hbac_ctx), hbac_ctx, &ret);
+        subreq = sdap_id_op_connect_send(hbac_ctx->sdap_op, hbac_ctx, &ret);
         if (!subreq) {
             DEBUG(1, ("sdap_id_op_connect_send failed: %d(%s).\n", ret, strerror(ret)));
             talloc_zfree(hbac_ctx->sdap_op);
@@ -280,6 +284,12 @@ static void hbac_clear_rule_data(struct hbac_ctx *hbac_ctx)
     talloc_zfree(hbac_ctx->rules);
 }
 
+/* Check whether the current HBAC request is processed in off-line mode */
+static inline bool hbac_ctx_is_offline(struct hbac_ctx *ctx)
+{
+    return ctx == NULL || ctx->sdap_op == NULL;
+}
+
 /* Check the step result code and continue, retry, get offline result or abort accordingly */
 static bool hbac_check_step_result(struct hbac_ctx *hbac_ctx, int ret)
 {
@@ -295,7 +305,7 @@ static bool hbac_check_step_result(struct hbac_ctx *hbac_ctx, int ret)
         return false;
     }
 
-    ret = sdap_id_op_done(hbac_ctx_sdap_id_op(hbac_ctx), ret, &dp_error);
+    ret = sdap_id_op_done(hbac_ctx->sdap_op, ret, &dp_error);
     if (ret != EOK) {
         if (dp_error == DP_ERR_OFFLINE) {
             /* switching to offline mode */
@@ -326,6 +336,7 @@ static void hbac_sysdb_save (struct tevent_req *req);
 
 static int hbac_get_host_info_step(struct hbac_ctx *hbac_ctx)
 {
+    struct be_ctx *be_ctx = be_req_get_be_ctx(hbac_ctx->be_req);
     const char *hostname;
     struct tevent_req *req;
 
@@ -339,11 +350,9 @@ static int hbac_get_host_info_step(struct hbac_ctx *hbac_ctx)
         hostname = dp_opt_get_string(hbac_ctx->ipa_options, IPA_HOSTNAME);
     }
 
-    req = ipa_host_info_send(hbac_ctx,
-                             hbac_ctx_ev(hbac_ctx),
-                             hbac_ctx_sysdb(hbac_ctx),
+    req = ipa_host_info_send(hbac_ctx, be_ctx->ev,
                              sdap_id_op_handle(hbac_ctx->sdap_op),
-                             hbac_ctx_sdap_id_ctx(hbac_ctx)->opts,
+                             hbac_ctx->sdap_ctx->opts,
                              hostname,
                              hbac_ctx->access_ctx->host_map,
                              hbac_ctx->access_ctx->hostgroup_map,
@@ -362,6 +371,7 @@ static void hbac_get_service_info_step(struct tevent_req *req)
     errno_t ret;
     struct hbac_ctx *hbac_ctx =
             tevent_req_callback_data(req, struct hbac_ctx);
+    struct be_ctx *be_ctx = be_req_get_be_ctx(hbac_ctx->be_req);
 
     ret = ipa_host_info_recv(req, hbac_ctx,
                              &hbac_ctx->host_count,
@@ -374,11 +384,9 @@ static void hbac_get_service_info_step(struct tevent_req *req)
     }
 
     /* Get services and service groups */
-    req = ipa_hbac_service_info_send(hbac_ctx,
-                                    hbac_ctx_ev(hbac_ctx),
-                                    hbac_ctx_sysdb(hbac_ctx),
+    req = ipa_hbac_service_info_send(hbac_ctx, be_ctx->ev,
                                     sdap_id_op_handle(hbac_ctx->sdap_op),
-                                    hbac_ctx_sdap_id_ctx(hbac_ctx)->opts,
+                                     hbac_ctx->sdap_ctx->opts,
                                     hbac_ctx->search_bases);
     if (req == NULL) {
         DEBUG(1,("Could not get service info\n"));
@@ -399,6 +407,7 @@ static void hbac_get_rule_info_step(struct tevent_req *req)
     const char *hostname;
     struct hbac_ctx *hbac_ctx =
             tevent_req_callback_data(req, struct hbac_ctx);
+    struct be_ctx *be_ctx = be_req_get_be_ctx(hbac_ctx->be_req);
 
     ret = ipa_hbac_service_info_recv(req, hbac_ctx,
                                      &hbac_ctx->service_count,
@@ -441,9 +450,9 @@ static void hbac_get_rule_info_step(struct tevent_req *req)
     /* Get the list of applicable rules */
     req = ipa_hbac_rule_info_send(hbac_ctx,
                                   hbac_ctx->get_deny_rules,
-                                  hbac_ctx_ev(hbac_ctx),
+                                  be_ctx->ev,
                                   sdap_id_op_handle(hbac_ctx->sdap_op),
-                                  hbac_ctx_sdap_id_ctx(hbac_ctx)->opts,
+                                  hbac_ctx->sdap_ctx->opts,
                                   hbac_ctx->search_bases,
                                   hbac_ctx->ipa_host);
     if (req == NULL) {
@@ -464,10 +473,9 @@ static void hbac_sysdb_save(struct tevent_req *req)
     bool in_transaction = false;
     struct hbac_ctx *hbac_ctx =
             tevent_req_callback_data(req, struct hbac_ctx);
-    struct sss_domain_info *domain = hbac_ctx_be(hbac_ctx)->domain;
-    struct sysdb_ctx *sysdb = hbac_ctx_sysdb(hbac_ctx);
+    struct be_ctx *be_ctx = be_req_get_be_ctx(hbac_ctx->be_req);
+    struct sss_domain_info *domain = be_ctx->domain;
     struct ldb_dn *base_dn;
-    struct be_ctx *be_ctx = hbac_ctx_be(hbac_ctx);
     struct ipa_access_ctx *access_ctx =
             talloc_get_type(be_ctx->bet_info[BET_ACCESS].pvt_bet_data,
                             struct ipa_access_ctx);
@@ -490,15 +498,15 @@ static void hbac_sysdb_save(struct tevent_req *req)
         /* Delete any rules in the sysdb so offline logins
          * are also denied.
          */
-        base_dn = sysdb_custom_subtree_dn(sysdb, tmp_ctx,
-                                          HBAC_RULES_SUBDIR);
+        base_dn = sysdb_custom_subtree_dn(domain->sysdb, tmp_ctx,
+                                          domain, HBAC_RULES_SUBDIR);
         if (base_dn == NULL) {
             talloc_free(tmp_ctx);
             ipa_access_reply(hbac_ctx, PAM_SYSTEM_ERR);
             return;
         }
 
-        ret = sysdb_delete_recursive(sysdb, base_dn, true);
+        ret = sysdb_delete_recursive(domain->sysdb, base_dn, true);
         talloc_free(tmp_ctx);
         if (ret != EOK) {
             DEBUG(1, ("sysdb_delete_recursive failed.\n"));
@@ -515,7 +523,7 @@ static void hbac_sysdb_save(struct tevent_req *req)
         return;
     }
 
-    ret = sysdb_transaction_start(sysdb);
+    ret = sysdb_transaction_start(domain->sysdb);
     if (ret != EOK) {
         DEBUG(0, ("Could not start transaction\n"));
         goto fail;
@@ -523,7 +531,7 @@ static void hbac_sysdb_save(struct tevent_req *req)
     in_transaction = true;
 
     /* Save the hosts */
-    ret = ipa_hbac_sysdb_save(sysdb, domain,
+    ret = ipa_hbac_sysdb_save(domain,
                               HBAC_HOSTS_SUBDIR, SYSDB_FQDN,
                               hbac_ctx->host_count, hbac_ctx->hosts,
                               HBAC_HOSTGROUPS_SUBDIR, SYSDB_NAME,
@@ -536,7 +544,7 @@ static void hbac_sysdb_save(struct tevent_req *req)
     }
 
     /* Save the services */
-    ret = ipa_hbac_sysdb_save(sysdb, domain,
+    ret = ipa_hbac_sysdb_save(domain,
                               HBAC_SERVICES_SUBDIR, IPA_CN,
                               hbac_ctx->service_count, hbac_ctx->services,
                               HBAC_SERVICEGROUPS_SUBDIR, IPA_CN,
@@ -548,7 +556,7 @@ static void hbac_sysdb_save(struct tevent_req *req)
         goto fail;
     }
     /* Save the rules */
-    ret = ipa_hbac_sysdb_save(sysdb, domain,
+    ret = ipa_hbac_sysdb_save(domain,
                               HBAC_RULES_SUBDIR, IPA_UNIQUE_ID,
                               hbac_ctx->rule_count,
                               hbac_ctx->rules,
@@ -559,7 +567,7 @@ static void hbac_sysdb_save(struct tevent_req *req)
         goto fail;
     }
 
-    ret = sysdb_transaction_commit(sysdb);
+    ret = sysdb_transaction_commit(domain->sysdb);
     if (ret != EOK) {
         DEBUG(SSSDBG_CRIT_FAILURE, ("Failed to commit transaction\n"));
         goto fail;
@@ -582,7 +590,7 @@ static void hbac_sysdb_save(struct tevent_req *req)
 
 fail:
     if (in_transaction) {
-        ret = sysdb_transaction_cancel(sysdb);
+        ret = sysdb_transaction_cancel(domain->sysdb);
         if (ret != EOK) {
             DEBUG(0, ("Could not cancel transaction\n"));
         }
@@ -592,6 +600,7 @@ fail:
 
 void ipa_hbac_evaluate_rules(struct hbac_ctx *hbac_ctx)
 {
+    struct be_ctx *be_ctx = be_req_get_be_ctx(hbac_ctx->be_req);
     errno_t ret;
     struct hbac_rule **hbac_rules;
     struct hbac_eval_req *eval_req;
@@ -599,7 +608,7 @@ void ipa_hbac_evaluate_rules(struct hbac_ctx *hbac_ctx)
     struct hbac_info *info;
 
     /* Get HBAC rules from the sysdb */
-    ret = hbac_get_cached_rules(hbac_ctx, hbac_ctx_sysdb(hbac_ctx),
+    ret = hbac_get_cached_rules(hbac_ctx, be_ctx->domain,
                                 &hbac_ctx->rule_count, &hbac_ctx->rules);
     if (ret != EOK) {
         DEBUG(1, ("Could not retrieve rules from the cache\n"));
@@ -644,7 +653,7 @@ void ipa_hbac_evaluate_rules(struct hbac_ctx *hbac_ctx)
 }
 
 errno_t hbac_get_cached_rules(TALLOC_CTX *mem_ctx,
-                              struct sysdb_ctx *sysdb,
+                              struct sss_domain_info *domain,
                               size_t *_rule_count,
                               struct sysdb_attrs ***_rules)
 {
@@ -680,7 +689,7 @@ errno_t hbac_get_cached_rules(TALLOC_CTX *mem_ctx,
         goto done;
     }
 
-    ret = sysdb_search_custom(tmp_ctx, sysdb, filter,
+    ret = sysdb_search_custom(tmp_ctx, domain->sysdb, domain, filter,
                               HBAC_RULES_SUBDIR, attrs,
                               &rule_count, &msgs);
     if (ret != EOK && ret != ENOENT) {
