@@ -511,6 +511,12 @@ errno_t sdap_get_user_primary_name(TALLOC_CTX *memctx,
                                    struct sss_domain_info *dom,
                                    const char **_user_name);
 
+errno_t sdap_get_netgroup_primary_name(TALLOC_CTX *memctx,
+                                       struct sdap_options *opts,
+                                       struct sysdb_attrs *attrs,
+                                       struct sss_domain_info *dom,
+                                       const char **_netgroup_name);
+
 errno_t sdap_set_config_options_with_rootdse(struct sysdb_attrs *rootdse,
                                              struct sdap_options *opts,
                                              struct sdap_domain *sdom);
