@@ -306,7 +306,7 @@ int sss_names_init(TALLOC_CTX *mem_ctx, struct confdb_ctx *cdb,
     }
 
     if (!fq_fmt) {
-        fq_fmt = talloc_strdup(tmpctx, "%1$s@%2$s");
+        fq_fmt = talloc_strdup(tmpctx, CONFDB_DEFAULT_FULL_NAME_FORMAT);
         if (!fq_fmt) {
             ret = ENOMEM;
             goto done;
@@ -394,7 +394,7 @@ int sss_parse_name_for_domains(TALLOC_CTX *memctx,
                                const char *default_domain,
                                const char *orig, char **domain, char **name)
 {
-    struct sss_domain_info *dom, *match;
+    struct sss_domain_info *dom, *match = NULL;
     char *rdomain, *rname;
     char *dmatch, *nmatch;
     char *candidate_name = NULL;
